import { DateMarker, EventSegUiInteractionState, Slicer, DateRange, DateEnv, Splitter, DateSpan, EventDef, DateComponent, ViewProps, DateProfile, DayTableCell, Hit, SlicedCoordRange, EventRangeProps, BaseComponent, DateProfileGenerator, DayTableModel } from '@fullcalendar/core/internal';
import { Duration } from '@fullcalendar/core';
import { createElement, Ref } from '@fullcalendar/core/preact';
import { RowConfig } from '@fullcalendar/daygrid/internal';

interface TimeGridRange {
    col: number;
    startDate: DateMarker;
    endDate: DateMarker;
    isStart: boolean;
    isEnd: boolean;
}
declare function organizeSegsByCol<S extends TimeGridRange>(segs: S[] | null, colCnt: number): S[][];
declare function splitInteractionByCol(ui: EventSegUiInteractionState<TimeGridRange> | null, colCnt: number): EventSegUiInteractionState<TimeGridRange>[];

declare class DayTimeColsSlicer extends Slicer<TimeGridRange, [DateRange[]]> {
    sliceRange(range: DateRange, dayRanges: DateRange[]): TimeGridRange[];
}

interface TimeSlatMeta {
    date: DateMarker;
    time: Duration;
    key: string;
    isoTimeStr: string;
    isLabeled: boolean;
}
declare function buildSlatMetas(slotMinTime: Duration, slotMaxTime: Duration, explicitLabelInterval: Duration | null, slotDuration: Duration, dateEnv: DateEnv): TimeSlatMeta[];

declare class AllDaySplitter extends Splitter {
    getKeyInfo(): {
        allDay: {};
        timed: {};
    };
    getKeysForDateSpan(dateSpan: DateSpan): string[];
    getKeysForEventDef(eventDef: EventDef): string[];
}

declare class TimeGridView extends DateComponent<ViewProps> {
    private createDayHeaderFormatter;
    private buildTimeColsModel;
    private buildDayRanges;
    private buildDateRowConfigs;
    private splitFgEventSegs;
    private splitBgEventSegs;
    private splitBusinessHourSegs;
    private splitNowIndicatorSegs;
    private splitDateSelectionSegs;
    private splitEventDrag;
    private splitEventResize;
    private allDaySplitter;
    private dayTableSlicer;
    private dayTimeColsSlicer;
    render(): createElement.JSX.Element;
}

interface TimeGridLayoutProps {
    labelId: string | undefined;
    labelStr: string | undefined;
    dateProfile: DateProfile;
    nowDate: DateMarker;
    todayRange: DateRange;
    cells: DayTableCell[];
    forPrint: boolean;
    isHitComboAllowed?: (hit0: Hit, hit1: Hit) => boolean;
    className: string;
    headerTiers: RowConfig<{
        text: string;
        isDisabled: boolean;
    }>[];
    fgEventSegs: (SlicedCoordRange & EventRangeProps)[];
    bgEventSegs: (SlicedCoordRange & EventRangeProps)[];
    businessHourSegs: (SlicedCoordRange & EventRangeProps)[];
    dateSelectionSegs: (SlicedCoordRange & EventRangeProps)[];
    eventDrag: EventSegUiInteractionState<SlicedCoordRange> | null;
    eventResize: EventSegUiInteractionState<SlicedCoordRange> | null;
    fgEventSegsByCol: (TimeGridRange & EventRangeProps)[][];
    bgEventSegsByCol: (TimeGridRange & EventRangeProps)[][];
    businessHourSegsByCol: (TimeGridRange & EventRangeProps)[][];
    nowIndicatorSegsByCol: TimeGridRange[][];
    dateSelectionSegsByCol: (TimeGridRange & EventRangeProps)[][];
    eventDragByCol: EventSegUiInteractionState<TimeGridRange>[];
    eventResizeByCol: EventSegUiInteractionState<TimeGridRange>[];
    eventSelection: string;
}
declare class TimeGridLayout extends BaseComponent<TimeGridLayoutProps> {
    private buildSlatMetas;
    private dayScrollerRef;
    private timeScrollerRef;
    private slatHeight?;
    private currentSlatCnt?;
    private scrollState;
    render(): createElement.JSX.Element;
    componentDidMount(): void;
    componentDidUpdate(prevProps: TimeGridLayoutProps): void;
    componentWillUnmount(): void;
    private handleSlatHeight;
    private resetScroll;
    private handleTimeScrollRequest;
    private handleTimeScrollEnd;
    private applyTimeScroll;
}

interface TimeGridWeekNumberProps {
    dateProfile: DateProfile;
    width: number | undefined;
    isLiquid: boolean;
    innerWidthRef?: Ref<number>;
    innerHeightRef?: Ref<number>;
}
declare class TimeGridWeekNumber extends BaseComponent<TimeGridWeekNumberProps> {
    private innerElRef;
    private disconnectInnerSize?;
    render(): createElement.JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
}

declare function buildTimeColsModel(dateProfile: DateProfile, dateProfileGenerator: DateProfileGenerator): DayTableModel;
declare function buildDayRanges(dayTableModel: DayTableModel, dateProfile: DateProfile, dateEnv: DateEnv): DateRange[];

export { AllDaySplitter, DayTimeColsSlicer, TimeGridLayout, TimeGridLayoutProps, TimeGridRange, TimeGridView, TimeGridWeekNumber, TimeSlatMeta, buildDayRanges, buildSlatMetas, buildTimeColsModel, organizeSegsByCol, splitInteractionByCol };
